// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kio_timeline_debug.h"

 namespace Baloo {
Q_LOGGING_CATEGORY(KIO_TIMELINE, "kf.kio.workers.timeline", QtWarningMsg)
} 
