/****************************************************************************
** Meta object code from reading C++ file 'fileindexscheduler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../baloo-6.26.0/src/file/fileindexscheduler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileindexscheduler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::FileIndexScheduler::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::FileIndexScheduler",
        "D-Bus Interface",
        "org.kde.baloo.scheduler",
        "stateChanged",
        "",
        "state",
        "indexNewFile",
        "file",
        "indexModifiedFile",
        "indexXAttrFile",
        "runnerFinished",
        "handleFileRemoved",
        "updateConfig",
        "scheduleIndexing",
        "startupFinished",
        "suspend",
        "resume",
        "getRemainingTime",
        "checkUnindexedFiles",
        "checkStaleIndexEntries",
        "getBatchSize",
        "powerManagementStatusChanged",
        "isOnBattery"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(int)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'indexNewFile'
        QtMocHelpers::SlotData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'indexModifiedFile'
        QtMocHelpers::SlotData<void(const QString &)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'indexXAttrFile'
        QtMocHelpers::SlotData<void(const QString &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'runnerFinished'
        QtMocHelpers::SlotData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleFileRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'updateConfig'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'scheduleIndexing'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startupFinished'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'suspend'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'resume'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'getRemainingTime'
        QtMocHelpers::SlotData<uint()>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'checkUnindexedFiles'
        QtMocHelpers::SlotData<void()>(18, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'checkStaleIndexEntries'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'getBatchSize'
        QtMocHelpers::SlotData<uint()>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'powerManagementStatusChanged'
        QtMocHelpers::SlotData<void(bool)>(21, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'state'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<FileIndexScheduler, qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Baloo::FileIndexScheduler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>.metaTypes,
    nullptr
} };

void Baloo::FileIndexScheduler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileIndexScheduler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->indexNewFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->indexModifiedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->indexXAttrFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->runnerFinished(); break;
        case 5: _t->handleFileRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->updateConfig(); break;
        case 7: _t->scheduleIndexing(); break;
        case 8: _t->startupFinished(); break;
        case 9: _t->suspend(); break;
        case 10: _t->resume(); break;
        case 11: { uint _r = _t->getRemainingTime();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->checkUnindexedFiles(); break;
        case 13: _t->checkStaleIndexEntries(); break;
        case 14: { uint _r = _t->getBatchSize();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->powerManagementStatusChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileIndexScheduler::*)(int )>(_a, &FileIndexScheduler::stateChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *Baloo::FileIndexScheduler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::FileIndexScheduler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileIndexSchedulerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::FileIndexScheduler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Baloo::FileIndexScheduler::stateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
