// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#ifndef ECM_QLOGGINGCATEGORY_BALOO_ENGINE_ENGINEDEBUG_H
#define ECM_QLOGGINGCATEGORY_BALOO_ENGINE_ENGINEDEBUG_H

#include <QLoggingCategory>
 namespace Baloo {
Q_DECLARE_LOGGING_CATEGORY(ENGINE)
} 

#endif
