/****************************************************************************
** Meta object code from reading C++ file 'pendingfilequeue.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../baloo-6.26.0/src/file/pendingfilequeue.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pendingfilequeue.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5Baloo16PendingFileQueueE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::PendingFileQueue::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::PendingFileQueue",
        "indexNewFile",
        "",
        "fileUrl",
        "indexModifiedFile",
        "indexXAttr",
        "removeFileIndex",
        "enqueue",
        "PendingFile",
        "file",
        "setTrackingTime",
        "seconds",
        "setMinimumTimeout",
        "setMaximumTimeout",
        "processCache",
        "currentTime",
        "processPendingFiles",
        "clearRecentlyEmitted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'indexNewFile'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'indexModifiedFile'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'indexXAttr'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'removeFileIndex'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'enqueue'
        QtMocHelpers::SlotData<void(const PendingFile &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setTrackingTime'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setMinimumTimeout'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setMaximumTimeout'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'processCache'
        QtMocHelpers::SlotData<void(const QTime &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QTime, 15 },
        }}),
        // Slot 'processPendingFiles'
        QtMocHelpers::SlotData<void(const QTime &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QTime, 15 },
        }}),
        // Slot 'clearRecentlyEmitted'
        QtMocHelpers::SlotData<void(const QTime &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QTime, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PendingFileQueue, qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Baloo::PendingFileQueue::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>.metaTypes,
    nullptr
} };

void Baloo::PendingFileQueue::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PendingFileQueue *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->indexNewFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->indexModifiedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->indexXAttr((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->removeFileIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->enqueue((*reinterpret_cast<std::add_pointer_t<PendingFile>>(_a[1]))); break;
        case 5: _t->setTrackingTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setMinimumTimeout((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setMaximumTimeout((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->processCache((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 9: _t->processPendingFiles((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        case 10: _t->clearRecentlyEmitted((*reinterpret_cast<std::add_pointer_t<QTime>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PendingFileQueue::*)(const QString & )>(_a, &PendingFileQueue::indexNewFile, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PendingFileQueue::*)(const QString & )>(_a, &PendingFileQueue::indexModifiedFile, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PendingFileQueue::*)(const QString & )>(_a, &PendingFileQueue::indexXAttr, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PendingFileQueue::*)(const QString & )>(_a, &PendingFileQueue::removeFileIndex, 3))
            return;
    }
}

const QMetaObject *Baloo::PendingFileQueue::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::PendingFileQueue::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo16PendingFileQueueE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::PendingFileQueue::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Baloo::PendingFileQueue::indexNewFile(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Baloo::PendingFileQueue::indexModifiedFile(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Baloo::PendingFileQueue::indexXAttr(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Baloo::PendingFileQueue::removeFileIndex(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
