/****************************************************************************
** Meta object code from reading C++ file 'monitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../baloo-6.26.0/src/qml/experimental/monitor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'monitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12BalooForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto BalooForeign::qt_create_metaobjectdata<qt_meta_tag_ZN12BalooForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BalooForeign",
        "QML.Element",
        "Global",
        "QML.Foreign",
        "Baloo",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<BalooForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject BalooForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12BalooForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12BalooForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12BalooForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN5Baloo7MonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::Monitor::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo7MonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::Monitor",
        "QML.Element",
        "auto",
        "newFileIndexed",
        "",
        "balooStateChanged",
        "totalFilesChanged",
        "remainingTimeChanged",
        "indexerStateChanged",
        "newFile",
        "filePath",
        "balooStarted",
        "slotIndexerStateChanged",
        "state",
        "toggleSuspendState",
        "startBaloo",
        "suspendState",
        "balooRunning",
        "totalFiles",
        "filesIndexed",
        "remainingTime",
        "stateString",
        "Baloo::IndexerState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newFileIndexed'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'balooStateChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'totalFilesChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remainingTimeChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indexerStateChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newFile'
        QtMocHelpers::SlotData<void(const QString &)>(9, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'balooStarted'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotIndexerStateChanged'
        QtMocHelpers::SlotData<void(int)>(12, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Method 'toggleSuspendState'
        QtMocHelpers::MethodData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'startBaloo'
        QtMocHelpers::MethodData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'filePath'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'suspendState'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'balooRunning'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'totalFiles'
        QtMocHelpers::PropertyData<uint>(18, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'filesIndexed'
        QtMocHelpers::PropertyData<uint>(19, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'remainingTime'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'stateString'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'state'
        QtMocHelpers::PropertyData<Baloo::IndexerState>(13, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Monitor, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN5Baloo7MonitorE[] = {
    QMetaObject::SuperData::link<Baloo::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Baloo::Monitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7MonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7MonitorE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN5Baloo7MonitorE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo7MonitorE_t>.metaTypes,
    nullptr
} };

void Baloo::Monitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Monitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newFileIndexed(); break;
        case 1: _t->balooStateChanged(); break;
        case 2: _t->totalFilesChanged(); break;
        case 3: _t->remainingTimeChanged(); break;
        case 4: _t->indexerStateChanged(); break;
        case 5: _t->newFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->balooStarted(); break;
        case 7: _t->slotIndexerStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->toggleSuspendState(); break;
        case 9: _t->startBaloo(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Monitor::*)()>(_a, &Monitor::newFileIndexed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Monitor::*)()>(_a, &Monitor::balooStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Monitor::*)()>(_a, &Monitor::totalFilesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Monitor::*)()>(_a, &Monitor::remainingTimeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Monitor::*)()>(_a, &Monitor::indexerStateChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->filePath(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->suspendState(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->m_balooRunning; break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->m_totalFiles; break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->m_filesIndexed; break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->remainingTime(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->stateString(); break;
        case 7: *reinterpret_cast<Baloo::IndexerState*>(_v) = _t->state(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2:
            if (QtMocHelpers::setProperty(_t->m_balooRunning, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->balooStateChanged();
            break;
        case 3:
            if (QtMocHelpers::setProperty(_t->m_totalFiles, *reinterpret_cast<uint*>(_v)))
                Q_EMIT _t->totalFilesChanged();
            break;
        case 4:
            if (QtMocHelpers::setProperty(_t->m_filesIndexed, *reinterpret_cast<uint*>(_v)))
                Q_EMIT _t->newFileIndexed();
            break;
        default: break;
        }
    }
}

const QMetaObject *Baloo::Monitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::Monitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7MonitorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::Monitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Baloo::Monitor::newFileIndexed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Baloo::Monitor::balooStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Baloo::Monitor::totalFilesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Baloo::Monitor::remainingTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Baloo::Monitor::indexerStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
