/****************************************************************************
** Meta object code from reading C++ file 'filecontentindexer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../baloo-6.26.0/src/file/filecontentindexer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filecontentindexer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5Baloo18FileContentIndexerE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::FileContentIndexer::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::FileContentIndexer",
        "D-Bus Interface",
        "org.kde.baloo.fileindexer",
        "startedIndexingFile",
        "",
        "filePath",
        "finishedIndexingFile",
        "finishedIndexingFile1",
        "Baloo::IndexResult::Changed",
        "changed",
        "Baloo::IndexResult::FileStatus",
        "fileStatus",
        "committedBatch",
        "time",
        "batchSize",
        "done",
        "registerMonitor",
        "QDBusMessage",
        "message",
        "unregisterMonitor",
        "currentFile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'startedIndexingFile'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'finishedIndexingFile'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'finishedIndexingFile1'
        QtMocHelpers::SignalData<void(const QString &, Baloo::IndexResult::Changed, Baloo::IndexResult::FileStatus)>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'committedBatch'
        QtMocHelpers::SignalData<void(uint, uint)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 13 }, { QMetaType::UInt, 14 },
        }}),
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'registerMonitor'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'unregisterMonitor'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentFile'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<FileContentIndexer, qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Baloo::FileContentIndexer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>.metaTypes,
    nullptr
} };

void Baloo::FileContentIndexer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileContentIndexer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->startedIndexingFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->finishedIndexingFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->finishedIndexingFile1((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Baloo::IndexResult::Changed>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Baloo::IndexResult::FileStatus>>(_a[3]))); break;
        case 3: _t->committedBatch((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 4: _t->done(); break;
        case 5: _t->registerMonitor((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 6: _t->unregisterMonitor((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Baloo::IndexResult::Changed >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Baloo::IndexResult::FileStatus >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileContentIndexer::*)(const QString & )>(_a, &FileContentIndexer::startedIndexingFile, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileContentIndexer::*)(const QString & )>(_a, &FileContentIndexer::finishedIndexingFile, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileContentIndexer::*)(const QString & , Baloo::IndexResult::Changed , Baloo::IndexResult::FileStatus )>(_a, &FileContentIndexer::finishedIndexingFile1, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileContentIndexer::*)(uint , uint )>(_a, &FileContentIndexer::committedBatch, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileContentIndexer::*)()>(_a, &FileContentIndexer::done, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->currentFile(); break;
        default: break;
        }
    }
}

const QMetaObject *Baloo::FileContentIndexer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::FileContentIndexer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo18FileContentIndexerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QRunnable"))
        return static_cast< QRunnable*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::FileContentIndexer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Baloo::FileContentIndexer::startedIndexingFile(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Baloo::FileContentIndexer::finishedIndexingFile(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Baloo::FileContentIndexer::finishedIndexingFile1(const QString & _t1, Baloo::IndexResult::Changed _t2, Baloo::IndexResult::FileStatus _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void Baloo::FileContentIndexer::committedBatch(uint _t1, uint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Baloo::FileContentIndexer::done()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
