/****************************************************************************
** Meta object code from reading C++ file 'kinotify.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../baloo-6.26.0/src/file/kinotify.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kinotify.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KInotifyE_t {};
} // unnamed namespace

template <> constexpr inline auto KInotify::qt_create_metaobjectdata<qt_meta_tag_ZN8KInotifyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KInotify",
        "accessed",
        "",
        "file",
        "attributeChanged",
        "closedWrite",
        "closedRead",
        "created",
        "isDir",
        "deleted",
        "modified",
        "moved",
        "oldName",
        "newName",
        "opened",
        "unmounted",
        "watchUserLimitReached",
        "path",
        "installedWatches",
        "addWatch",
        "WatchEvents",
        "modes",
        "WatchFlags",
        "flags",
        "removeWatch",
        "slotEvent",
        "slotClearCookies"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accessed'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'attributeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'closedWrite'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'closedRead'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'created'
        QtMocHelpers::SignalData<void(const QString &, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 8 },
        }}),
        // Signal 'deleted'
        QtMocHelpers::SignalData<void(const QString &, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 8 },
        }}),
        // Signal 'modified'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'moved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'opened'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'unmounted'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'watchUserLimitReached'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'installedWatches'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addWatch'
        QtMocHelpers::SlotData<bool(const QString &, WatchEvents, WatchFlags)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'addWatch'
        QtMocHelpers::SlotData<bool(const QString &, WatchEvents)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'removeWatch'
        QtMocHelpers::SlotData<bool(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'slotEvent'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotClearCookies'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KInotify, qt_meta_tag_ZN8KInotifyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KInotify::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KInotifyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KInotifyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8KInotifyE_t>.metaTypes,
    nullptr
} };

void KInotify::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KInotify *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accessed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->attributeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->closedWrite((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->closedRead((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->created((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->deleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->modified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->moved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->opened((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->unmounted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->watchUserLimitReached((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->installedWatches(); break;
        case 12: { bool _r = _t->addWatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WatchEvents>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WatchFlags>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->addWatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WatchEvents>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->removeWatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->slotEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotClearCookies(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::accessed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::attributeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::closedWrite, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::closedRead, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & , bool )>(_a, &KInotify::created, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & , bool )>(_a, &KInotify::deleted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::modified, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & , const QString & )>(_a, &KInotify::moved, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::opened, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::unmounted, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)(const QString & )>(_a, &KInotify::watchUserLimitReached, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KInotify::*)()>(_a, &KInotify::installedWatches, 11))
            return;
    }
}

const QMetaObject *KInotify::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KInotify::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KInotifyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KInotify::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void KInotify::accessed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KInotify::attributeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KInotify::closedWrite(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KInotify::closedRead(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KInotify::created(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void KInotify::deleted(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KInotify::modified(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KInotify::moved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void KInotify::opened(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void KInotify::unmounted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void KInotify::watchUserLimitReached(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void KInotify::installedWatches()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
