// This file is generated by kconfig_compiler_kf6 from baloosettings.kcfg.
// All changes you do to this file will be lost.

#include "baloosettings.h"

BalooSettings::BalooSettings( QObject *parent )
  : KCoreConfigSkeleton( QStringLiteral( "baloofilerc" ) )
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "Basic Settings" ) );

  KCoreConfigSkeleton::ItemBool  *itemIndexingEnabled;
  itemIndexingEnabled = new KCoreConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Indexing-Enabled" ), mIndexingEnabled, true );
  addItem( itemIndexingEnabled, QStringLiteral( "indexingEnabled" ) );

  setCurrentGroup( QStringLiteral( "General" ) );

  KCoreConfigSkeleton::ItemBool  *itemIndexHiddenFolders;
  itemIndexHiddenFolders = new KCoreConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "index hidden folders" ), mIndexHiddenFolders, false );
  addItem( itemIndexHiddenFolders, QStringLiteral( "indexHiddenFolders" ) );
  KCoreConfigSkeleton::ItemInt  *itemDbVersion;
  itemDbVersion = new KCoreConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "dbVersion" ), mDbVersion, 0 );
  addItem( itemDbVersion, QStringLiteral( "dbVersion" ) );
  KCoreConfigSkeleton::ItemBool  *itemOnlyBasicIndexing;
  itemOnlyBasicIndexing = new KCoreConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "only basic indexing" ), mOnlyBasicIndexing, false );
  addItem( itemOnlyBasicIndexing, QStringLiteral( "onlyBasicIndexing" ) );
  KCoreConfigSkeleton::ItemBool  *itemDisableInitialUpdate;
  itemDisableInitialUpdate = new KCoreConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "disable initial update" ), mDisableInitialUpdate, true );
  addItem( itemDisableInitialUpdate, QStringLiteral( "disableInitialUpdate" ) );
  KCoreConfigSkeleton::ItemPathList  *itemExcludedFolders;
  itemExcludedFolders = new KCoreConfigSkeleton::ItemPathList( currentGroup(), QStringLiteral( "exclude folders" ), mExcludedFolders, QStringList() );
  addItem( itemExcludedFolders, QStringLiteral( "excludedFolders" ) );
  KCoreConfigSkeleton::ItemPathList  *itemFolders;
  itemFolders = new KCoreConfigSkeleton::ItemPathList( currentGroup(), QStringLiteral( "folders" ), mFolders, QStringList() << QDir::homePath() );
  addItem( itemFolders, QStringLiteral( "folders" ) );
  KCoreConfigSkeleton::ItemStringList  *itemExcludedMimetypes;
  itemExcludedMimetypes = new KCoreConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "exclude mimetypes" ), mExcludedMimetypes, Baloo::defaultExcludeMimetypes() );
  addItem( itemExcludedMimetypes, QStringLiteral( "excludedMimetypes" ) );
  KCoreConfigSkeleton::ItemStringList  *itemExcludedFilters;
  itemExcludedFilters = new KCoreConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "exclude filters" ), mExcludedFilters, Baloo::defaultExcludeFilterList() );
  addItem( itemExcludedFilters, QStringLiteral( "excludedFilters" ) );
  KCoreConfigSkeleton::ItemInt  *itemExcludedFiltersVersion;
  itemExcludedFiltersVersion = new KCoreConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "exclude filters version" ), mExcludedFiltersVersion, 0 );
  addItem( itemExcludedFiltersVersion, QStringLiteral( "excludedFiltersVersion" ) );
}

bool BalooSettings::defaultIndexingEnabledValue_helper() const
{

  return true;
}

bool BalooSettings::defaultIndexHiddenFoldersValue_helper() const
{

  return false;
}

int BalooSettings::defaultDbVersionValue_helper() const
{

  return 0;
}

bool BalooSettings::defaultOnlyBasicIndexingValue_helper() const
{

  return false;
}

bool BalooSettings::defaultDisableInitialUpdateValue_helper() const
{

  return true;
}

QStringList BalooSettings::defaultExcludedFoldersValue_helper() const
{

  return QStringList();
}

QStringList BalooSettings::defaultFoldersValue_helper() const
{

  return QStringList() << QDir::homePath();
}

QStringList BalooSettings::defaultExcludedMimetypesValue_helper() const
{

  return Baloo::defaultExcludeMimetypes();
}

QStringList BalooSettings::defaultExcludedFiltersValue_helper() const
{

  return Baloo::defaultExcludeFilterList();
}

int BalooSettings::defaultExcludedFiltersVersionValue_helper() const
{

  return 0;
}

BalooSettings::~BalooSettings()
{
}

