/****************************************************************************
** Meta object code from reading C++ file 'indexerstate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../baloo-6.26.0/src/engine/indexerstate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'indexerstate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5BalooE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::qt_create_metaobjectdata<qt_meta_tag_ZN5BalooE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo",
        "IndexerState",
        "Idle",
        "Suspended",
        "FirstRun",
        "NewFiles",
        "ModifiedFiles",
        "XAttrFiles",
        "ContentIndexing",
        "UnindexedFileCheck",
        "StaleIndexEntriesClean",
        "LowPowerIdle",
        "Unavailable",
        "Startup"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'IndexerState'
        QtMocHelpers::EnumData<IndexerState>(1, 1, QMC::EnumFlags{}).add({
            {    2, IndexerState::Idle },
            {    3, IndexerState::Suspended },
            {    4, IndexerState::FirstRun },
            {    5, IndexerState::NewFiles },
            {    6, IndexerState::ModifiedFiles },
            {    7, IndexerState::XAttrFiles },
            {    8, IndexerState::ContentIndexing },
            {    9, IndexerState::UnindexedFileCheck },
            {   10, IndexerState::StaleIndexEntriesClean },
            {   11, IndexerState::LowPowerIdle },
            {   12, IndexerState::Unavailable },
            {   13, IndexerState::Startup },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN5BalooE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN5BalooE =
    Baloo::qt_create_metaobjectdata<qt_meta_tag_ZN5BalooE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN5BalooE =
    qt_staticMetaObjectContent_ZN5BalooE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN5BalooE =
    qt_staticMetaObjectContent_ZN5BalooE.relocatingData;

Q_CONSTINIT const QMetaObject Baloo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN5BalooE.stringdata,
    qt_staticMetaObjectStaticContent_ZN5BalooE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN5BalooE.metaTypes,
    nullptr
} };

QT_WARNING_POP
