/****************************************************************************
** Meta object code from reading C++ file 'commandpipe.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../baloo-6.26.0/src/file/extractor/commandpipe.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'commandpipe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::Private::ControllerPipe::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::Private::ControllerPipe",
        "urlStarted",
        "",
        "url",
        "urlProcessed",
        "updated",
        "Baloo::IndexResult::FileStatus",
        "status",
        "batchFinished",
        "processStatusData"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlStarted'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'urlProcessed'
        QtMocHelpers::SignalData<void(const QString &, bool, Baloo::IndexResult::FileStatus)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'batchFinished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'processStatusData'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControllerPipe, qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Baloo::Private::ControllerPipe::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>.metaTypes,
    nullptr
} };

void Baloo::Private::ControllerPipe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControllerPipe *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->urlProcessed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Baloo::IndexResult::FileStatus>>(_a[3]))); break;
        case 2: _t->batchFinished(); break;
        case 3: _t->processStatusData(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Baloo::IndexResult::FileStatus >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ControllerPipe::*)(const QString & )>(_a, &ControllerPipe::urlStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControllerPipe::*)(const QString & , bool , Baloo::IndexResult::FileStatus )>(_a, &ControllerPipe::urlProcessed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControllerPipe::*)()>(_a, &ControllerPipe::batchFinished, 2))
            return;
    }
}

const QMetaObject *Baloo::Private::ControllerPipe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::Private::ControllerPipe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private14ControllerPipeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::Private::ControllerPipe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Baloo::Private::ControllerPipe::urlStarted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Baloo::Private::ControllerPipe::urlProcessed(const QString & _t1, bool _t2, Baloo::IndexResult::FileStatus _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Baloo::Private::ControllerPipe::batchFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t {};
} // unnamed namespace

template <> constexpr inline auto Baloo::Private::WorkerPipe::qt_create_metaobjectdata<qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Baloo::Private::WorkerPipe",
        "newDocumentIds",
        "",
        "QList<quint64>",
        "ids",
        "inputEnd",
        "processIdData"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newDocumentIds'
        QtMocHelpers::SignalData<void(const QVector<quint64> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'inputEnd'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'processIdData'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkerPipe, qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Baloo::Private::WorkerPipe::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>.metaTypes,
    nullptr
} };

void Baloo::Private::WorkerPipe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkerPipe *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newDocumentIds((*reinterpret_cast<std::add_pointer_t<QList<quint64>>>(_a[1]))); break;
        case 1: _t->inputEnd(); break;
        case 2: _t->processIdData(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<quint64> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkerPipe::*)(const QVector<quint64> & )>(_a, &WorkerPipe::newDocumentIds, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkerPipe::*)()>(_a, &WorkerPipe::inputEnd, 1))
            return;
    }
}

const QMetaObject *Baloo::Private::WorkerPipe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Baloo::Private::WorkerPipe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5Baloo7Private10WorkerPipeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Baloo::Private::WorkerPipe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Baloo::Private::WorkerPipe::newDocumentIds(const QVector<quint64> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Baloo::Private::WorkerPipe::inputEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
