/****************************************************************************
** Meta object code from reading C++ file 'baloosettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../baloosettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'baloosettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13BalooSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto BalooSettings::qt_create_metaobjectdata<qt_meta_tag_ZN13BalooSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BalooSettings",
        "indexingEnabledChanged",
        "",
        "indexHiddenFoldersChanged",
        "dbVersionChanged",
        "onlyBasicIndexingChanged",
        "disableInitialUpdateChanged",
        "excludedFoldersChanged",
        "foldersChanged",
        "excludedMimetypesChanged",
        "excludedFiltersChanged",
        "excludedFiltersVersionChanged",
        "indexingEnabled",
        "isIndexingEnabledImmutable",
        "defaultIndexingEnabledValue",
        "indexHiddenFolders",
        "isIndexHiddenFoldersImmutable",
        "defaultIndexHiddenFoldersValue",
        "dbVersion",
        "isDbVersionImmutable",
        "defaultDbVersionValue",
        "onlyBasicIndexing",
        "isOnlyBasicIndexingImmutable",
        "defaultOnlyBasicIndexingValue",
        "disableInitialUpdate",
        "isDisableInitialUpdateImmutable",
        "defaultDisableInitialUpdateValue",
        "excludedFolders",
        "isExcludedFoldersImmutable",
        "defaultExcludedFoldersValue",
        "folders",
        "isFoldersImmutable",
        "defaultFoldersValue",
        "excludedMimetypes",
        "isExcludedMimetypesImmutable",
        "defaultExcludedMimetypesValue",
        "excludedFilters",
        "isExcludedFiltersImmutable",
        "defaultExcludedFiltersValue",
        "excludedFiltersVersion",
        "isExcludedFiltersVersionImmutable",
        "defaultExcludedFiltersVersionValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'indexingEnabledChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indexHiddenFoldersChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dbVersionChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onlyBasicIndexingChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disableInitialUpdateChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'excludedFoldersChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'foldersChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'excludedMimetypesChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'excludedFiltersChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'excludedFiltersVersionChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'indexingEnabled'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isIndexingEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultIndexingEnabledValue'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'indexHiddenFolders'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isIndexHiddenFoldersImmutable'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultIndexHiddenFoldersValue'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dbVersion'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isDbVersionImmutable'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultDbVersionValue'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'onlyBasicIndexing'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isOnlyBasicIndexingImmutable'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultOnlyBasicIndexingValue'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'disableInitialUpdate'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isDisableInitialUpdateImmutable'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultDisableInitialUpdateValue'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'excludedFolders'
        QtMocHelpers::PropertyData<QStringList>(27, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isExcludedFoldersImmutable'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultExcludedFoldersValue'
        QtMocHelpers::PropertyData<QStringList>(29, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'folders'
        QtMocHelpers::PropertyData<QStringList>(30, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isFoldersImmutable'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultFoldersValue'
        QtMocHelpers::PropertyData<QStringList>(32, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'excludedMimetypes'
        QtMocHelpers::PropertyData<QStringList>(33, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'isExcludedMimetypesImmutable'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultExcludedMimetypesValue'
        QtMocHelpers::PropertyData<QStringList>(35, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'excludedFilters'
        QtMocHelpers::PropertyData<QStringList>(36, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'isExcludedFiltersImmutable'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultExcludedFiltersValue'
        QtMocHelpers::PropertyData<QStringList>(38, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'excludedFiltersVersion'
        QtMocHelpers::PropertyData<int>(39, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'isExcludedFiltersVersionImmutable'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultExcludedFiltersVersionValue'
        QtMocHelpers::PropertyData<int>(41, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BalooSettings, qt_meta_tag_ZN13BalooSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BalooSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<KCoreConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BalooSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BalooSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13BalooSettingsE_t>.metaTypes,
    nullptr
} };

void BalooSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BalooSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->indexingEnabledChanged(); break;
        case 1: _t->indexHiddenFoldersChanged(); break;
        case 2: _t->dbVersionChanged(); break;
        case 3: _t->onlyBasicIndexingChanged(); break;
        case 4: _t->disableInitialUpdateChanged(); break;
        case 5: _t->excludedFoldersChanged(); break;
        case 6: _t->foldersChanged(); break;
        case 7: _t->excludedMimetypesChanged(); break;
        case 8: _t->excludedFiltersChanged(); break;
        case 9: _t->excludedFiltersVersionChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::indexingEnabledChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::indexHiddenFoldersChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::dbVersionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::onlyBasicIndexingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::disableInitialUpdateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::excludedFoldersChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::foldersChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::excludedMimetypesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::excludedFiltersChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (BalooSettings::*)()>(_a, &BalooSettings::excludedFiltersVersionChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->indexingEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isIndexingEnabledImmutable(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->defaultIndexingEnabledValue(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->indexHiddenFolders(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isIndexHiddenFoldersImmutable(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->defaultIndexHiddenFoldersValue(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->dbVersion(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isDbVersionImmutable(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->defaultDbVersionValue(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->onlyBasicIndexing(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isOnlyBasicIndexingImmutable(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->defaultOnlyBasicIndexingValue(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->disableInitialUpdate(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isDisableInitialUpdateImmutable(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->defaultDisableInitialUpdateValue(); break;
        case 15: *reinterpret_cast<QStringList*>(_v) = _t->excludedFolders(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isExcludedFoldersImmutable(); break;
        case 17: *reinterpret_cast<QStringList*>(_v) = _t->defaultExcludedFoldersValue(); break;
        case 18: *reinterpret_cast<QStringList*>(_v) = _t->folders(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isFoldersImmutable(); break;
        case 20: *reinterpret_cast<QStringList*>(_v) = _t->defaultFoldersValue(); break;
        case 21: *reinterpret_cast<QStringList*>(_v) = _t->excludedMimetypes(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->isExcludedMimetypesImmutable(); break;
        case 23: *reinterpret_cast<QStringList*>(_v) = _t->defaultExcludedMimetypesValue(); break;
        case 24: *reinterpret_cast<QStringList*>(_v) = _t->excludedFilters(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->isExcludedFiltersImmutable(); break;
        case 26: *reinterpret_cast<QStringList*>(_v) = _t->defaultExcludedFiltersValue(); break;
        case 27: *reinterpret_cast<int*>(_v) = _t->excludedFiltersVersion(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->isExcludedFiltersVersionImmutable(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->defaultExcludedFiltersVersionValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setIndexingEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setIndexHiddenFolders(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setDbVersion(*reinterpret_cast<int*>(_v)); break;
        case 9: _t->setOnlyBasicIndexing(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setDisableInitialUpdate(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setExcludedFolders(*reinterpret_cast<QStringList*>(_v)); break;
        case 18: _t->setFolders(*reinterpret_cast<QStringList*>(_v)); break;
        case 21: _t->setExcludedMimetypes(*reinterpret_cast<QStringList*>(_v)); break;
        case 24: _t->setExcludedFilters(*reinterpret_cast<QStringList*>(_v)); break;
        case 27: _t->setExcludedFiltersVersion(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *BalooSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BalooSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BalooSettingsE_t>.strings))
        return static_cast<void*>(this);
    return KCoreConfigSkeleton::qt_metacast(_clname);
}

int BalooSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KCoreConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void BalooSettings::indexingEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void BalooSettings::indexHiddenFoldersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void BalooSettings::dbVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void BalooSettings::onlyBasicIndexingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void BalooSettings::disableInitialUpdateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void BalooSettings::excludedFoldersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void BalooSettings::foldersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void BalooSettings::excludedMimetypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void BalooSettings::excludedFiltersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void BalooSettings::excludedFiltersVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
